/*****************************************************************
*
* PROJECT:        Swiss Pairing Test Program
*
*                 Copyright (C) 1996 Robert Rozycki
*                 SwissPerfect/SPAR General Public Licence
*
* FILE:           log.cpp
*
* DESCRIPTION:    Debug logging functions
*
* AUTHOR:         Robert Rozycki   rozycki@perth.dialix.oz.au
*
* PORTABILITY:    ANSI C++
*
* $Revision: 1.1 $
*
* $Date: 1996/02/04 09:46:22 $
*
* $Log: log.cpp $
// Revision 1.1  1996/02/04  09:46:22  robert
// Initial revision
//
*
*******************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include "log.h"

//*********************** INTERFACE ********************************

///////////////////////////////////////////////////////////////////
//
// Constructor
//
DebugLog_c::DebugLog_c() : MsgMngr_c()
{
   LogFile[ MAX_LOGFILE_NAME ] = 0;
   LogDebugLev = DispDebugLev = 0;
   strcpy( LogFile, "sp3.log" );
}

///////////////////////////////////////////////////////////////////
//
// Sets Debug Levels ( to disk and to screen )
//
void DebugLog_c::SetDebugLevs( int log_debug, int disp_debug )
{
   LogDebugLev  = log_debug;
   DispDebugLev = disp_debug;
}

///////////////////////////////////////////////////////////////////
//
// Sets log file name
//
void DebugLog_c::SetLogFileName( char *name )
{
   strncpy( LogFile, name, MAX_LOGFILE_NAME );
}

///////////////////////////////////////////////////////////////////
//
//  Logs a debug message to screen and/or log file depending on
//  the priority passed and current debug log levels
//
int DebugLog_c::WriteToLog( int debug_level, char *msg, ... )
{
   char      buf[ 512 ];
   int     len;
   va_list args;

   if( LogDebugLev < debug_level  &&  DispDebugLev < debug_level)
      return(0);

   va_start( args, msg );
   vsprintf( buf, msg, args );
   va_end( args );

   /* append a newline if required */
   len = strlen(buf);
   if( buf[ len-1 ] != '\n' )
   {
      buf[ len ] = '\n';
      len++;
      buf[ len ]   = '\0';
   }

   if( LogDebugLev >= debug_level)
   {
      WriteLogToDisk( buf, len );
   }

   if( DispDebugLev >= debug_level)
   {
      buf[ len-1 ] = '\0';
      puts( buf );
   }
   return(0);
}

//*********************** IMPLEMENTATION ******************************

///////////////////////////////////////////////////////////////////
//
//  Writes debug message to disk
//

int DebugLog_c::WriteLogToDisk( void *buf, int len )
{
   FILE *fd;

   fd = fopen( LogFile, "a" );
   if( fd != NULL )
   {
       if( fwrite( buf, 1, len, fd ) <= 0 )
          printf( "WriteLogToDisk(): cannot write to file" );
       fclose( fd );
   }
   else
   {
      printf( "WriteLogToDisk(): cannot open log file" );
      return(-1);
   }
   return(0);
}

