/*****************************************************************
*
* PROJECT:     Swiss Perfect 3 GNU
*
*              Copyright (C) 1996 Robert Rozycki
*              SwissPerfect/SPAR General Public Licence
*
* FILE:        sws_base.cpp
*
* DESCRIPTION: Implementation of the base class for Swiss pairing
*
* AUTHOR:      Robert Rozycki   rozycki@perth.dialix.oz.au 
*
* PORTABILITY: ANSI C++
*
* $Revision: 1.1 $
*
* $Date: 1996/02/04 09:52:28 $
*
* $Log: sws_base.cpp $
// Revision 1.1  1996/02/04  09:52:28  robert
// Initial revision
//
*
*******************************************************************/

#include "l_types.h"
#include "sws_base.h"

////////////////////////////////////////////////////
// MUST be called before mapping
// (Uses DrawPool[] ordering for correct sorting)
// higher list number (ie arithmetically lower) determines placing
// in Round Table
void SwissBase_c::SortRoundTab ( PAIR *rtab )
{
   int i, hi1, hi2;
   int tmp;
   int found = TRUE;

   while( found == TRUE )
   {
      found = FALSE;
      for( i = 1; i < PairTablesToBeMade-1; i++ )
      {
         hi1 = min( rtab[i].left, rtab[i].right );
         hi2 = min( rtab[i+1].left, rtab[i+1].right );
         if( hi2 == 0 )
            continue;
         if( hi1 > hi2 )
         {
            tmp = rtab[i].left;
            rtab[i].left = rtab[i+1].left;
            rtab[i+1].left = tmp;

            tmp = rtab[i].right;
            rtab[i].right = rtab[i+1].right;
            rtab[i+1].right = tmp;

            found = TRUE;
         }
      }
  }
}

void SwissBase_c::SwapPair( PAIR *rtab, int pair_no )
{
   int tmp;

   tmp = rtab[ pair_no ].left;
   rtab[ pair_no ].left = rtab[ pair_no ].right;
   rtab[ pair_no ].right = tmp;
}

void SwissBase_c::FirstRoundCols( PAIR *rtab )
{
   int i, tmp, pairs;

   pairs = PlayersToBePaired >> 1;
   for( i = 1; i <= pairs; i++ )
   {
      if( ( i & 0x01 ) == 0 )
      {
         tmp = rtab[ i ].left;
         rtab[ i ].left = rtab[ i ].right;
         rtab[ i ].right = tmp;
      }
   }
}

int SwissBase_c::AssignColours( PAIR *pairing )
{
   int i, w_pref, b_pref, pairs, result;
   int left, right, left_score, right_score, left_higher;

   if( RoundNoToPair == 1 )
   {
      FirstRoundCols( pairing );
      return( SUCCESS );
   }
   pairs = PlayersToBePaired >> 1;
   for( i = 1; i <= pairs; i++ )
   {
      left  = pairing[ i ].left;
      w_pref = GetColPref( left );
      right  = pairing[ i ].right;
      b_pref = GetColPref( right );
      if( w_pref != b_pref )
      {
         if( w_pref > b_pref ) // swap
            SwapPair( pairing, i );
      }
      else
      {
         result = ColourHistory( RoundNoToPair-1, left, right );
         if( result > 0 )
         {
            SwapPair( pairing, i );
         }
         if( result == 0 )
         {
            // find the higher ranked player
            left_score =  GetPlayerScore( left );
            right_score = GetPlayerScore( right );
            if( left_score == right_score )
            {
               if( left < right )
                  left_higher = TRUE;
               else
                  left_higher = FALSE;
            }
            else
            {
               if( left_score > right_score )
                  left_higher = TRUE;
               else
                  left_higher = FALSE;
            }
            // give the higher ranked player expected colour
            // REMEMBER! w_pref == b_pref
            if( left_higher && w_pref > 0 )
                SwapPair( pairing, i );
            if( !left_higher && w_pref < 0 )
                SwapPair( pairing, i );
         }
      }
   }
  return 0;
}

void SwissBase_c::SaveFinalPairing( PAIR *pairing )
{
   int i, tables;

   tables = PairTablesToBeMade;
   for( i = 1; i <= tables; i++ )
   {
      pairing[ i ].left  = FinalPairing[ i ].left;
      pairing[ i ].right = FinalPairing[ i ].right;
   }
}

