/*****************************************************************
*
* PROJECT:        Swiss Pairing Test Program
*
*                 Copyright (C) 1996 Robert Rozycki
*                 SwissPerfect/SPAR General Public Licence
*
* FILE:           sws_data.h
*
* DESCRIPTION:
*
* AUTHOR:         Robert Rozycki    rozycki@perth.dialix.oz.au
*
* PORTABILITY:    ANSI C++
*
* $Revision: 1.2 $
*
* $Date: 1996/02/04 09:48:15 $
*
* $Log: sws_data.h $
 * Revision 1.2  1996/02/04  09:48:15  robert
 * Internet release Feb 1996
 *
*
*******************************************************************/

#ifndef _SWS_DATA_H
#define _SWS_DATA_H

#define MAX_MAPVECTORS   100

#include "log.h"
#include "sws_pair.h"

class SwissData_c : public SwissGrPair_c
{
protected:
  int  *OpponsCount;
  int  *ColPref;        /* holds colour preference for each player */
  char *UpFloats;       /* holds upfloat status for each player */
  char *DownFloats;     /* holds downfloat status for each player */
  char *Byes;
  int **AllOppons;
  int   PlayersCount;   /* number of players defined in the data file */
  int  *MapVectors[ MAX_MAPVECTORS + 1 ];         /* mapping */
  int   MVector;
  int   GroupsCount;
//  int   GroupMarks[ MAX_ROUNDS * 2 + 1 ];
  int   GroupMarks[ 255 ];

public:
  SwissData_c( MsgMngr_c * );
  ~SwissData_c( void );

/* Interface functions for retrieving data */
  int  CanPlay( int player1, int player2 );
  int  GetColPref( int player )
      { return( ColPref[ player ] ); };
  char GetUpfStatus( int player )
      { return( UpFloats[ player ] ); };
  char GetDnfStatus( int player )
      { return( DownFloats[ player ] ); };
  char GetByeStatus( int player )
      { return( Byes[ player ] ); };
  int  GetGroup( int group, int& first, int& last );
  int  GetPlayersCount( void )
      { return PlayersCount; };
  int  GetRoundPlayers( void )
      { return PlayersCount; };
  int  GetOpponCount( int player );

  int MakePairing( PAIR *pairing, int players_count, int, int );

/* Interface functions for loading data */
  void IncPlayersCount( void )
      { PlayersCount++; };
  int  LoadOppon( int player, int OppNo );
  int  AddPlayer( int player );
  int  GetOppon( int player, int OppIdx );
  int  MapPlayer( int player, int map_option = TRUE );
  int  DemapPlayer( int player, int map_option = TRUE );
  void DemapRoundTab( PAIR *pairing, int players_count );
  void LoadUpfStatus( int player, char status )
      { UpFloats[ player ] = status; };
  void LoadDnfStatus( int player, char status )
      { DownFloats[ player ] = status; };
  void LoadByeStatus( int player, char status )
      { Byes[ player ] = status; };
  void LoadColPref( int player, int col_pref )
      { ColPref[ player ] = col_pref; };
  int *ReallocOpponVector( int player, int new_count );
  int *AllocOpponVector( int player, int oppon_count, int forbidden );
  int  AllocTables( void );
  void NewGroupMark( void );

protected:
/* private functions */
  void ResetVars( void );
};

#endif


