/*****************************************************************
*
* PROJECT:        Swiss Perfect 3  Gnu Public License
*
* FILE:           sws_pair.h
*
* DESCRIPTION:    Header file for sws_pair.cpp
*
* AUTHOR:
*
* PORTABILITY:    ANSI C++
* 
* $Revision: 1.1 $
*
* $Date: 1996/02/04 09:53:38 $
*
* $Log: sws_pair.h $
 * Revision 1.1  1996/02/04  09:53:38  robert
 * Initial revision
 *
*
*******************************************************************/
#ifndef _SWS_PAIR_H
#define _SWS_PAIR_H

#include "l_msg.h"
#include "sws_ctrl.h"

#define  LEFT              0
#define  RIGHT             1

/* Colour compatibilities */

#define  INCOMPATIBLE     -1
#define  FULL_COMPAT       0
#define  GOOD_COMPAT       1
#define  BAD_COMPAT        2
#define  VBAD_COMPAT       3

enum { PAIRS_ONLY, PAIR_COLOURS, ALL_CRITS };

typedef struct {
  int left;     // left player
  int right;    // right player
  int compat;   // colour compatibility
  int exch;     // exchange difference if any
} PAIR_PARAMS;

typedef struct {
   int no;
   int qual;  // FL_GOOD, FL_BAD, FL_VERY_BAD
} TxFloat_t;

typedef struct {
   int stuck;          // stuck player no
   int level;          // stuck level
   int rx_float;       // which rx_float can unstick
   PAIR_PARAMS *ptr;   // NOT USED currently
} Stuck_t;

typedef struct {
   int state;
   int last_group;
   int allow_exch;
   int allowed_ccl;
   int max_noncols;
   int prevail_col_pref;
   int rx_float_again;
   int stuck;
   int stuck_count;
   int stuck_level;
   Stuck_t tx_float_stuck;
   Stuck_t rx_float_stuck;
   Stuck_t rx_float_oppon_stuck;
   PAIR_PARAMS *best_pairing;
} PairingStatus_t;

typedef struct {
   int oppon;
   int level;
   int side;
} LevelSide_t;

enum {NO_EXCHANGES_OPTIMAL, FIND_ANY_PAIRING, EXCHANGES_BEST_COLS,
      NO_EXCHANGES_BEST, RX_FLOATS_PAIRING };
enum { FROM_SCRATCH, IMPROVE };
enum { NO_EXCHANGES, ALLOW_EXCHANGES };


class SwissGrPair_c : public SwissCtrl_c
{
public:
  SwissGrPair_c( MsgMngr_c *);
  ~SwissGrPair_c() {};
  int PairGroup( PAIRED_VEC *pGroup, int is_last_group );

private:
   int    TotalGroupSize;
   int    GroupSize;
   int    RxFloats;
   int    MidGroup;
   int    MaxPairs;
   int    FixedPairs;
   int    IsTxFloat;
   int    IsLastGroup;

   int   *TempUsed;
   int   *pGroup;

   PAIR_PARAMS    *GrPairing, *BestPairing;
   PairingStatus_t PairStat;

// Pairing Control
   int  PairingControl();
   int  PairStateOptimal();
   int  PairStateFindAny();
   int  PairStateExchBestCols();
   int  PairStateNoExchBest();

   int  PairRemainder( int modify_lev, int allow_exch, int max_non_cols );
   int  PairRxFloats( int modify_lev, int max_noncols );

   int  CalcMaxNonCols();
   int  CalcMidGroup();
   int  SelectTxFloat( int curr_float = 0, int same_qual = FALSE );
   void InitSubGroupPairing( int clear_level );

   void SaveToBestPairing();
   void RetrieveFromBestPairing();

   int  RelaxColReqs();
   void ClearBottomPairings( int clear_level, int bot_level, int both = TRUE );
   void LoadRxFloats();

// Pairing Subgroups
   int  PairSubGroup( int top_lev, int bot_lev, int exch, int mod_lev,
                      int max_noncols );
   int  GetNextFree( int current = 0 );
   int  GetNextOppon( int level, int strict_col );
   int  GetNextRxOppon( int level, int strict_col );
   LevelSide_t
        ReplaceWithOpponFromAbove( int lev, int strict, int exch, int top=1 );

   void LoadLeftPlayers( int top_level, int bot_level );
   void UnpairLevels( int top_lev, int bot_lev );

   void WouldModifyFloatsHelp( int ord, int level, int strict_col );
   void ClearStuckData();

   int  CountNonCols( int level );
   int  CompatLevel( int ord1, int ord2 );

   int  ReplaceRxFloatOppon( int start_lev );
   int  ColourCompatLevel( int ord1, int ord2 );
   int  SameHalf( int ord1, int ord2, int mid_group );
   int  MultipleByes();
   void AppendFloaters( int pairs_made, PAIR_PARAMS * );
   void AppendFixedPairs( int pairs_made, PAIR_PARAMS *pair_table );

// Removing players from group
   int  RemoveNoHopers();
   void RemoveRxFloatFromGroup( int ord );
   void RemoveStuckFromGroup( int ord, int always = FALSE );
   void RemoveStuckPlayer();
   void RemovePlayer( int pos, int group_end );
   int  RemoveFixedPairs();

// Memory Allocation
   int  AllocGroupPairingTables();
   void ClearGroupPairingTables();
   void FreeGroupPairingTables( void );
};
#endif
